from dataclasses import dataclass


@dataclass
class Aircraft:
    """Aircraft are used to describe what passengers will fly in for a given trip"""

    id: str
    iata_code: str
    name: str

    @classmethod
    def from_json(cls, json: dict):
        """Construct a class instance from a JSON response."""
        return cls(
            id=json["id"],
            iata_code=json["iata_code"],
            name=json["name"],
        )



from .fixtures import fixture


def test_create_webhook(requests_mock):
    url = "air/webhooks"
    with fixture("create-webhook", url, requests_mock.post, 201) as client:
        webhook = (
            client.webhooks.create()
            .url("https://www.example.com:4000/webhooks")
            .events(["order.created", "order.updated"])
            .execute()
        )

        assert webhook.id == "sev_0000A3tQSmKyqOrcySrGbo"
        assert webhook.active
        assert webhook.events == ["order.created", "order.updated"]
        assert webhook.secret == "QKfUULLQh+8SegYmIsF6kA=="
        assert webhook.url == "https://www.example.com:4000/webhooks"


def test_update_webhook(requests_mock):
    url = "air/webhooks/some-id"
    with fixture("update-webhook", url, requests_mock.patch, 200) as client:
        webhook = client.webhooks.update("some-id").active(False).execute()

        assert webhook.id == "sev_0000A3tQSmKyqOrcySrGbo"
        assert not webhook.active
        assert webhook.events == ["order.created", "order.updated"]
        assert webhook.url == "https://www.example.com:4000/webhooks"


def test_ping_webhook(requests_mock):
    url = "air/webhooks/some-id/actions/ping"
    with fixture(
        "ping-webhook", url, requests_mock.post, 204, with_response=False
    ) as client:
        webhook = client.webhooks.ping("some-id")

        assert webhook is None



from .fixtures import fixture


def test_get_airline_by_id(requests_mock):
    url = "air/airlines/id"
    with fixture("get-airline-by-id", url, requests_mock.get, 200) as client:
        airline = client.airlines.get("id")
        assert airline.id == "aln_00001876aqC8c5umZmrRds"
        assert airline.name == "British Airways"
        assert airline.iata_code == "BA"


def test_get_airline_by_id_without_iata_code(requests_mock):
    url = "air/airlines/id"
    with fixture(
        "get-airline-without-iata-code", url, requests_mock.get, 200
    ) as client:
        airline = client.airlines.get("id")
        assert airline.id == "some-id"
        assert airline.name == "Duffel Airways"
        assert airline.iata_code is None


def test_get_airlines(requests_mock):
    # We need a way to ensure pagination finished in a mocking environment
    end_pagination_url = (
        "http://someaddress/air/airlines?limit=50"
        + "&after=g2wAAAACbQAAABBBZXJvbWlzdC1LaGFya2l2bQAAAB%3D"
    )
    end_pagination_response = {"meta": {"after": None}, "data": []}
    requests_mock.get(
        end_pagination_url, complete_qs=True, json=end_pagination_response
    )

    url = "air/airlines?limit=50"
    with fixture("get-airlines", url, requests_mock.get, 200) as client:
        paginated_airlines = client.airlines.list()
        airlines = list(paginated_airlines)
        assert len(airlines) == 1
        airline = airlines[0]
        assert airline.id == "aln_00001876aqC8c5umZmrRds"
        assert airline.name == "British Airways"
        assert airline.iata_code == "BA"



from .aircraft import Aircraft
from .airline import Airline
from .airport import Airport, City, Place, Refund
from .loyalty_programme_account import LoyaltyProgrammeAccount
from .offer import (
    Offer,
    OfferConditionChangeBeforeDeparture,
    OfferConditionRefundBeforeDeparture,
    OfferPassenger,
)
from .offer_request import OfferRequest
from .order import (
    Order,
    OrderConditionChangeBeforeDeparture,
    OrderConditionRefundBeforeDeparture,
)
from .order_cancellation import OrderCancellation
from .order_change import OrderChange
from .order_change_offer import OrderChangeOffer
from .order_change_request import OrderChangeRequest
from .payment import Payment
from .payment_intent import PaymentIntent
from .seat_map import SeatMap
from .webhook import Webhook

__all__ = [
    "Aircraft",
    "Airline",
    "Airport",
    "City",
    "LoyaltyProgrammeAccount",
    "Place",
    "Offer",
    "OfferPassenger",
    "OfferConditionChangeBeforeDeparture",
    "OfferConditionRefundBeforeDeparture",
    "OfferRequest",
    "Order",
    "OrderConditionChangeBeforeDeparture",
    "OrderConditionRefundBeforeDeparture",
    "OrderCancellation",
    "OrderChange",
    "OrderChangeOffer",
    "OrderChangeRequest",
    "Payment",
    "PaymentIntent",
    "Refund",
    "SeatMap",
    "Webhook",
]
